const express = require('express');
const path = require('path');
const cors = require('cors');
const helmet = require('helmet');
const crawlerService = require('./services/crawlerService');

// 处理 punycode 弃用警告
process.removeAllListeners('warning');
process.on('warning', (warning) => {
  if (warning.name === 'DeprecationWarning' && warning.message.includes('punycode')) {
    return;
  }
  console.warn(warning);
});

const app = express();

// 安全中间件
app.use(helmet({
  crossOriginResourcePolicy: { policy: "cross-origin" }
}));

// CORS配置
app.use(cors({
  origin: '*', // 允许所有来源访问，生产环境建议设置具体的域名
  methods: ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS'],
  allowedHeaders: ['Content-Type', 'Authorization']
}));

// 解析 JSON 请求体
app.use(express.json());
// 解析 URL 编码的请求体
app.use(express.urlencoded({ extended: true }));

// 设置静态文件目录
const screenshotsDir = path.join(process.cwd(), 'screenshots');
app.use('/screenshots', express.static(screenshotsDir));

// 获取本机IP地址
const os = require('os');
function getLocalIP() {
  const interfaces = os.networkInterfaces();
  for (const name of Object.keys(interfaces)) {
    for (const iface of interfaces[name]) {
      // 跳过内部IP和非IPv4地址
      if (iface.family === 'IPv4' && !iface.internal) {
        return iface.address;
      }
    }
  }
  return '0.0.0.0'; // 默认监听所有接口
}

// 添加请求日志中间件
app.use((req, res, next) => {
  console.log(`${new Date().toISOString()} - ${req.method} ${req.url}`);
  next();
});

// 商品信息API路由
app.get('/api/product', async (req, res) => {
  try {
    const { url, needScreenshot = false, includeAllSkus = false } = req.query;
    console.log('收到商品信息请求:', { url, needScreenshot, includeAllSkus });
    
    if (!url) {
      console.log('请求缺少URL参数');
      return res.status(400).json({
        success: false,
        message: '请提供商品URL',
        example: '/api/product?url=https://www.amazon.co.jp/dp/XXXXX'
      });
    }

    console.log('开始爬取商品信息:', url);
    const data = await crawlerService.crawlProductInfo(url, needScreenshot === 'true', includeAllSkus === 'true');
    console.log('爬取完成:', data);
    
    res.json({
      success: true,
      data
    });
  } catch (error) {
    console.error('获取商品信息失败:', error);
    res.status(500).json({
      success: false,
      message: '获取商品信息失败',
      error: error.message
    });
  }
});

// 添加健康检查端点
app.get('/health', (req, res) => {
  res.json({ status: 'ok', timestamp: new Date().toISOString() });
});

// 404 处理
app.use((req, res) => {
  res.status(404).json({
    success: false,
    message: '接口不存在',
    path: req.path
  });
});

// 错误处理中间件
app.use((err, req, res, next) => {
  console.error('服务器错误:', err);
  res.status(500).json({
    success: false,
    message: '服务器内部错误',
    error: process.env.NODE_ENV === 'development' ? err.message : undefined
  });
});

// 设置端口
const PORT = process.env.PORT || 8991;
const HOST = '0.0.0.0'; // 监听所有网络接口

// 启动服务器
app.listen(PORT, HOST, () => {
  const localIP = getLocalIP();
  console.log('服务器配置:');
  console.log(`- 监听地址: ${HOST}:${PORT}`);
  console.log(`- 本地访问: http://localhost:${PORT}`);
  console.log(`- 局域网访问: http://${localIP}:${PORT}`);
  console.log(`- 健康检查: http://${localIP}:${PORT}/health`);
  console.log('\nAPI 使用示例:');
  console.log('http://localhost:8991/api/product?url=https://www.amazon.co.jp/dp/XXXXX');
  console.log('可选参数:');
  console.log('- needScreenshot=true');
  console.log('- includeAllSkus=true');
}); 