const { chromium } = require('playwright');
const path = require('path');
const fs = require('fs').promises;
const FormData = require('form-data');
const fetch = require('node-fetch');

/**
 * Amazon 商品信息爬虫服务（Playwright）
 */
class CrawlerService {
  constructor() {
    // 设置静态文件访问的基础URL
    this.UPLOAD_URL = 'https://apibase.sohomall.jp/uploaders?scene=goods';
  }

  async initBrowser() {
    return await chromium.launch({ headless: true });
  }

  async createScreenshotDir() {
    const dir = path.join(process.cwd(), 'screenshots');
    await fs.mkdir(dir, { recursive: true });
    return dir;
  }

  /**
   * 获取单个 SKU 信息（含优惠券扣减）
   */
  async getSingleSkuInfo(page) {
    // 添加控制台日志监听
    page.on('console', msg => console.log('Browser Console:', msg.text()));
    
    await page.waitForTimeout(500);
    let couponValue = 0;
    const couponTrigger = await page.$('.a-declarative[data-action="a-modal"], .couponLabelText');
    if (couponTrigger) {
      try {
        await couponTrigger.click();
        await page.waitForTimeout(500);
      } catch {}
      try {
        const couponText = await page.$eval('.couponLabelText', el => el.textContent.trim());
        const m = couponText.match(/¥\s*([\d,]+)/);
        couponValue = m ? parseInt(m[1].replace(/,/g, '')) : 0;
        console.log('Found coupon value:', couponValue);
      } catch {}
      // 尝试关闭弹窗
      try { await page.click('button.a-modal-close', { timeout: 1000 }); } catch { await page.keyboard.press('Escape'); }
    }

    return await page.evaluate(couponValue => {
      const title = document.querySelector('#productTitle')?.textContent.trim() || null;
      let priceText = document.querySelector('span.a-price > span.a-offscreen')?.textContent.trim()
        || document.querySelector('#priceblock_dealprice')?.textContent.trim()
        || document.querySelector('#priceblock_saleprice')?.textContent.trim()
        || document.querySelector('#priceblock_ourprice')?.textContent.trim()
        || null;
      if (priceText?.includes('ポイント')) priceText = priceText.split('ポイント')[0].trim();
      priceText = priceText.replace('￥', '');
      priceText = priceText.replace('JP¥', '');
      const m = priceText?.match(/\s*([\d,]+)/);
      let priceVal = m ? parseInt(m[1].replace(/,/g, '')) : null;
      if (priceVal != null) priceVal -= couponValue;
      console.log('priceText', priceText);
      console.log('priceVal', priceVal);
      console.log('couponValue', couponValue);
      const price = priceVal != null ? `${priceVal.toLocaleString()}` : priceText;
      const url = window.location.href;
      const asin = url.match(/\/dp\/([A-Z0-9]{10})/)?.[1] || null;
      return { title, price, sku: asin, url, remark: `Original Price: JP¥${priceText} Coupon Price: JP¥${couponValue}` };
    }, couponValue);
  }

  /**
   * 获取所有 SKU 组合信息（笛卡尔积方式）
   */
  async getAllSkuInfo(page) {
    // 等待 SKU 分组加载
    await page.waitForSelector('.a-cardui-body #twister-plus-inline-twister > .a-section');
    const groupEls = await page.$$('.a-cardui-body #twister-plus-inline-twister > .a-section');
    const groups = [];
    for (const groupEl of groupEls) {
      const btns = await groupEl.$$('.a-button-inner .a-button-input');
      if (btns.length) groups.push(btns);
    }
    if (!groups.length) return [await this.getSingleSkuInfo(page)];

    // 生成笛卡尔积组合
    const cartesian = (arr1, arr2) => arr1.flatMap(a => arr2.map(b => [...a, b]));
    let combos = groups[0].map(b => [b]);
    for (let i = 1; i < groups.length; i++) combos = cartesian(combos, groups[i]);

    const results = [];
    for (const combo of combos) {
      // 依次点击每个维度按钮
      for (const btn of combo) {
        await btn.click();
        await page.waitForLoadState('networkidle');
      }
      // 获取当前组合信息
      const info = await this.getSingleSkuInfo(page);
      // 添加 variants 字段
      info.variants = await Promise.all(
        combo.map(btn => btn.getAttribute('aria-label') || btn.getAttribute('title'))
      );
      results.push(info);
    }
    return results;
  }

  /**
   * 将图片转换为base64
   * @param {string} imagePath - 图片路径
   * @returns {Promise<string>} base64字符串
   */
  async convertImageToBase64(imagePath) {
    try {
      const imageBuffer = await fs.readFile(imagePath);
      return `data:image/png;base64,${imageBuffer.toString('base64')}`;
    } catch (error) {
      console.error('转换图片到base64失败:', error);
      return null;
    }
  }

  /**
   * 上传图片到服务器
   * @param {string} imagePath - 图片路径
   * @returns {Promise<string>} 上传后的图片URL
   */
  async uploadImage(imagePath) {
    try {
      const form = new FormData();
      form.append('file', await fs.readFile(imagePath), {
        filename: path.basename(imagePath),
        contentType: 'image/png'
      });

      const response = await fetch(this.UPLOAD_URL, {
        method: 'POST',
        body: form
      });

      if (!response.ok) {
        throw new Error(`上传失败: ${response.statusText}`);
      }

      const result = await response.json();
      return result.url; // 假设服务器返回的数据中包含url字段
    } catch (error) {
      console.error('上传图片失败:', error);
      return null;
    }
  }

  /**
   * 主方法：抓取商品信息
   */
  async crawlProductInfo(url, needScreenshot = false, includeAllSkus = false) {
    const browser = await this.initBrowser();
    const context = await browser.newContext({ locale: 'ja-JP', userAgent: 'Mozilla/5.0' });
    const page = await context.newPage();
    await page.goto(url.split('?')[0], { waitUntil: 'networkidle' });

    const data = includeAllSkus
      ? await this.getAllSkuInfo(page)
      : [await this.getSingleSkuInfo(page)];

    if (needScreenshot) {
      const dir = await this.createScreenshotDir();
      const filename = `${Date.now()}.png`;
      const shot = path.join(dir, filename);
      await page.screenshot({ path: shot, fullPage: true });
      
      // 上传图片并获取URL
      const imageUrl = await this.uploadImage(shot);
      
      // 更新数据，添加图片URL
      data.forEach(item => {
        item.screenshotUrl = imageUrl;
      });

      // 删除临时文件
      try {
        await fs.unlink(shot);
      } catch (error) {
        console.error('删除临时截图文件失败:', error);
      }
    }

    await browser.close();
    return data;
  }
}

module.exports = new CrawlerService();
